<?php
namespace Controller;

use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\DevOps\Util\Environment;

class ProductVersion extends \Generated\Controller\ProductVersion {

    private $environment;

    public function __construct(Manager $dataSourceManager, Environment $environment) {
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        try {
            $dataSet     = $request->getDataSet();
            $productVersionId = $dataSet->getRows()[0]['ID'];
            
            InstanceProvider::getService()->getProductVersion()->validateRemoveLTSVersion($productVersionId);
        
            
            $showMessage = Utilities::verifyShowMessage($dataSet);
            parent::save($request, $response, $showMessage);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $showMessage = Utilities::verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    public function findLTSProductVersion(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $version    = InstanceProvider::getService()->getProductVersion()->findLTSProductVersion($productId, $conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('findLTSProductVersion', $version));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function minimumProductVersion(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $version  = Utilities::getValueInFilterDataSourceByName($conditions, 'VERSION_ID', true);
            $productVersionProduct    = InstanceProvider::getService()->getProductVersion()->minimumProductVersion($version);
            $response->addDataSet(new DataSet('minimumProductVersion', $productVersionProduct));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function minimumVersion(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $productVersionProduct = InstanceProvider::getService()->getProductVersion()->minimumVersion($productId, $conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('minimumVersion', $productVersionProduct));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findModulesConfiguration(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $modulesConfiguration    = InstanceProvider::getService()->getProductVersion()->findModulesConfiguration($conditions);
            $response->addDataSet(new DataSet('findModulesConfiguration', $modulesConfiguration));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveModulesConfiguration(DTO\Request\Dataset $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            InstanceProvider::getService()->getProductVersion()->saveModulesConfiguration($rows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveProductVersionProducts(DTO\Request\Dataset $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            $productId = $rows['PRODUCT_ID'];
            $depProductVersionId = $rows['DEP_PRODUCT_VERSION_ID'];
            $productVersionId = $rows['PRODUCT_VERSION_ID'];
            $id = isset($rows['ID']) ? $rows['ID'] : null;
            $id = InstanceProvider::getService()->getProductVersion()->saveProductVersionProducts($id, $productId, $productVersionId, $depProductVersionId);
            $response->addDataSet(new DataSet('productVersionProducts', array($id)));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

     public function removeProductVersionProducts(DTO\Request $request, DTO\Response $response) {
        try {
             
            $minimumVersion = $request->getRow();
            $minimumVersionId = $minimumVersion['ID'];
             InstanceProvider::getService()->getProductVersion()->removeProductVersionProducts($minimumVersionId);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getCurrentBuildFromVersion(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions       = $request->getFilterCriteria()->getConditions();
            $productVersionId = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
            $currentBuild     = InstanceProvider::getService()->getProductVersion()->getCurrentBuildFromVersion($productVersionId);
            $response->addDataSet(new DataSet('getCurrentBuildFromVersion', array($currentBuild)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findLTSProductVersionToRemove(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $filterDate = Utilities::getValueInFilterDataSourceByName($conditions, 'FILTER_DATE', true);
            $version    = InstanceProvider::getService()->getProductVersion()->findLTSProductVersionToRemove($productId, $filterDate);
            $response->addDataSet(new DataSet('findLTSProductVersionToRemove', $version));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeLTSFromVersions(DTO\Request\Dataset $request, DTO\Response $response) {
        try {
            InstanceProvider::getService()->getProductVersion()->removeLTSFromVersions($request->getDataSet()->getRows());
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}